@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -62,19 +62,19 @@ static const char *storages[] = {
 	"SM",
 	"ME",
 	"MT",
+	"SR",
+	"BM",
 };
 
-#define SM_STORE 0
-#define ME_STORE 1
-#define MT_STORE 2
-
 struct sms_data {
 	int store;
 	int incoming;
 	int retries;
+	gboolean expect_sr;
 	gboolean cnma_enabled;
 	char *cnma_ack_pdu;
 	int cnma_ack_pdu_len;
+	guint timeout_source;
 	GAtChat *chat;
 	unsigned int vendor;
 };
@@ -83,6 +83,7 @@ struct cpms_request {
 	struct ofono_sms *sms;
 	int store;
 	int index;
+	gboolean expect_sr;
 };
 
 static void at_csca_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -91,7 +92,6 @@ static void at_csca_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	ofono_sms_sca_set_cb_t cb = cbd->cb;
 	struct ofono_error error;
 
-	dump_response("csca_set_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	cb(&error, cbd->data);
@@ -108,7 +108,7 @@ static void at_csca_set(struct ofono_sms *sms,
 	if (!cbd)
 		goto error;
 
-	sprintf(buf, "AT+CSCA=\"%s\",%d", sca->number, sca->type);
+	snprintf(buf, sizeof(buf), "AT+CSCA=\"%s\",%d", sca->number, sca->type);
 
 	if (g_at_chat_send(data->chat, buf, csca_prefix,
 				at_csca_set_cb, cbd, g_free) > 0)
@@ -130,7 +130,6 @@ static void at_csca_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_phone_number sca;
 	const char *number;
 
-	dump_response("at_csca_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok) {
@@ -149,15 +148,16 @@ static void at_csca_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	if (number[0] == '+') {
 		number = number + 1;
 		sca.type = 145;
-	} else
+	} else {
 		sca.type = 129;
+	}
 
 	strncpy(sca.number, number, OFONO_MAX_PHONE_NUMBER_LENGTH);
 	sca.number[OFONO_MAX_PHONE_NUMBER_LENGTH] = '\0';
 
 	g_at_result_iter_next_number(&iter, &sca.type);
 
-	ofono_debug("csca_query_cb: %s, %d", sca.number, sca.type);
+	DBG("csca_query_cb: %s, %d", sca.number, sca.type);
 
 	cb(&error, &sca, cbd->data);
 
@@ -195,7 +195,6 @@ static void at_cmgs_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_error error;
 	int mr;
 
-	dump_response("at_cmgs_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok) {
@@ -211,7 +210,7 @@ static void at_cmgs_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	if (!g_at_result_iter_next_number(&iter, &mr))
 		goto err;
 
-	ofono_debug("Got MR: %d", mr);
+	DBG("Got MR: %d", mr);
 
 	cb(&error, mr, cbd->data);
 	return;
@@ -233,12 +232,12 @@ static void at_cmgs(struct ofono_sms *sms, unsigned char *pdu, int pdu_len,
 		goto error;
 
 	if (mms) {
-		sprintf(buf, "AT+CMMS=%d", mms);
+		snprintf(buf, sizeof(buf), "AT+CMMS=%d", mms);
 		g_at_chat_send(data->chat, buf, none_prefix,
 				NULL, NULL, NULL);
 	}
 
-	len = sprintf(buf, "AT+CMGS=%d\r", tpdu_len);
+	len = snprintf(buf, sizeof(buf), "AT+CMGS=%d\r", tpdu_len);
 	encode_hex_own_buf(pdu, pdu_len, 0, buf+len);
 
 	if (g_at_chat_send(data->chat, buf, cmgs_prefix,
@@ -280,44 +279,55 @@ static gboolean at_parse_pdu_common(GAtResult *result, const char *prefix,
 	return TRUE;
 }
 
+static inline void at_ack_delivery(struct ofono_sms *sms)
+{
+	struct sms_data *data = ofono_sms_get_data(sms);
+	char buf[256];
+
+	/* We must acknowledge the PDU using CNMA */
+	if (data->cnma_ack_pdu)
+		snprintf(buf, sizeof(buf), "AT+CNMA=1,%d\r%s",
+				data->cnma_ack_pdu_len, data->cnma_ack_pdu);
+	else /* Should be a safe fallback */
+		snprintf(buf, sizeof(buf), "AT+CNMA=0");
+
+	g_at_chat_send(data->chat, buf, none_prefix, at_cnma_cb, NULL, NULL);
+}
+
 static void at_cds_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_sms *sms = user_data;
-	struct sms_data *data = ofono_sms_get_data(sms);
-	int pdulen;
-	const char *pdu;
-	char buf[256];
+	long pdu_len;
+	int tpdu_len;
+	const char *hexpdu;
+	unsigned char pdu[176];
 
-	dump_response("at_cds_notify", TRUE, result);
-
-	if (!at_parse_pdu_common(result, "+CDS:", &pdu, &pdulen)) {
+	if (!at_parse_pdu_common(result, "+CDS:", &hexpdu, &tpdu_len)) {
 		ofono_error("Unable to parse CDS notification");
 		return;
 	}
 
-	ofono_debug("Got new Status-Report PDU via CDS: %s, %d", pdu, pdulen);
+	if (strlen(hexpdu) > sizeof(pdu) * 2) {
+		ofono_error("Bad PDU length in CDS notification");
+		return;
+	}
 
-	/* We must acknowledge the PDU using CNMA */
-	if (data->cnma_ack_pdu)
-		sprintf(buf, "AT+CNMA=1,%d\r%s", data->cnma_ack_pdu_len,
-				data->cnma_ack_pdu);
-	else
-		sprintf(buf, "AT+CNMA=0"); /* Should be a safe fallback */
+	DBG("Got new Status-Report PDU via CDS: %s, %d", hexpdu, tpdu_len);
 
-	g_at_chat_send(data->chat, buf, none_prefix, at_cnma_cb, NULL, NULL);
+	/* Decode pdu and notify about new SMS status report */
+	decode_hex_own_buf(hexpdu, -1, &pdu_len, 0, pdu);
+	ofono_sms_status_notify(sms, pdu, pdu_len, tpdu_len);
+
+	at_ack_delivery(sms);
 }
 
 static void at_cmt_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_sms *sms = user_data;
-	struct sms_data *data = ofono_sms_get_data(sms);
 	const char *hexpdu;
 	long pdu_len;
 	int tpdu_len;
 	unsigned char pdu[176];
-	char buf[256];
-
-	dump_response("at_cmt_notify", TRUE, result);
 
 	if (!at_parse_pdu_common(result, "+CMT:", &hexpdu, &tpdu_len)) {
 		ofono_error("Unable to parse CMT notification");
@@ -329,33 +339,24 @@ static void at_cmt_notify(GAtResult *result, gpointer user_data)
 		return;
 	}
 
-	ofono_debug("Got new SMS Deliver PDU via CMT: %s, %d",
-			hexpdu, tpdu_len);
+	DBG("Got new SMS Deliver PDU via CMT: %s, %d", hexpdu, tpdu_len);
 
 	decode_hex_own_buf(hexpdu, -1, &pdu_len, 0, pdu);
 	ofono_sms_deliver_notify(sms, pdu, pdu_len, tpdu_len);
 
-	/* We must acknowledge the PDU using CNMA */
-	if (data->cnma_ack_pdu)
-		sprintf(buf, "AT+CNMA=1,%d\r%s", data->cnma_ack_pdu_len,
-				data->cnma_ack_pdu);
-	else
-		sprintf(buf, "AT+CNMA=0"); /* Should be a safe fallback */
-
-	g_at_chat_send(data->chat, buf, none_prefix, at_cnma_cb, NULL, NULL);
+	at_ack_delivery(sms);
 }
 
 static void at_cmgr_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_sms *sms = user_data;
+	struct sms_data *data = ofono_sms_get_data(sms);
 	GAtResultIter iter;
 	const char *hexpdu;
 	unsigned char pdu[176];
 	long pdu_len;
 	int tpdu_len;
 
-	dump_response("at_cmgr_notify", TRUE, result);
-
 	g_at_result_iter_init(&iter, result);
 
 	if (!g_at_result_iter_next(&iter, "+CMGR:"))
@@ -375,10 +376,14 @@ static void at_cmgr_notify(GAtResult *result, gpointer user_data)
 	if (strlen(hexpdu) > sizeof(pdu) * 2)
 		goto err;
 
-	ofono_debug("Got PDU: %s, with len: %d", hexpdu, tpdu_len);
+	DBG("Got PDU: %s, with len: %d", hexpdu, tpdu_len);
 
 	decode_hex_own_buf(hexpdu, -1, &pdu_len, 0, pdu);
-	ofono_sms_deliver_notify(sms, pdu, pdu_len, tpdu_len);
+
+	if (data->expect_sr)
+		ofono_sms_status_notify(sms, pdu, pdu_len, tpdu_len);
+	else
+		ofono_sms_deliver_notify(sms, pdu, pdu_len, tpdu_len);
 	return;
 
 err:
@@ -397,7 +402,7 @@ static void at_cmgd_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		ofono_error("Unable to delete received SMS");
 }
 
-static void at_cmti_cpms_cb(gboolean ok, GAtResult *result, gpointer user_data)
+static void at_cmgr_cpms_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cpms_request *req = user_data;
 	struct ofono_sms *sms = req->sms;
@@ -405,50 +410,25 @@ static void at_cmti_cpms_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	char buf[128];
 
 	if (!ok) {
-		ofono_error("Received CMTI, but CPMS request failed");
+		ofono_error("Received CMTI/CDSI, but CPMS request failed");
 		return;
 	}
 
 	data->store = req->store;
+	data->expect_sr = req->expect_sr;
 
-	sprintf(buf, "AT+CMGR=%d", req->index);
+	snprintf(buf, sizeof(buf), "AT+CMGR=%d", req->index);
 	g_at_chat_send(data->chat, buf, none_prefix, at_cmgr_cb, NULL, NULL);
 
 	/* We don't buffer SMS on the SIM/ME, send along a CMGD as well */
-	sprintf(buf, "AT+CMGD=%d", req->index);
+	snprintf(buf, sizeof(buf), "AT+CMGD=%d", req->index);
 	g_at_chat_send(data->chat, buf, none_prefix, at_cmgd_cb, NULL, NULL);
 }
 
-static void at_cmti_notify(GAtResult *result, gpointer user_data)
+static void at_send_cmgr_cpms(struct ofono_sms *sms, int store, int index,
+				gboolean expect_sr)
 {
-	struct ofono_sms *sms = user_data;
 	struct sms_data *data = ofono_sms_get_data(sms);
-	const char *strstore;
-	int store;
-	GAtResultIter iter;
-	int index;
-
-	dump_response("at_cmti_notify", TRUE, result);
-
-	g_at_result_iter_init(&iter, result);
-
-	if (!g_at_result_iter_next(&iter, "+CMTI:"))
-		goto err;
-
-	if (!g_at_result_iter_next_string(&iter, &strstore))
-		goto err;
-
-	if (!strcmp(strstore, "ME"))
-		store = ME_STORE;
-	else if (!strcmp(strstore, "SM"))
-		store = SM_STORE;
-	else
-		goto err;
-
-	if (!g_at_result_iter_next_number(&iter, &index))
-		goto err;
-
-	ofono_debug("Got a CMTI indication at %s, index: %d", strstore, index);
 
 	if (store == data->store) {
 		struct cpms_request req;
@@ -456,8 +436,9 @@ static void at_cmti_notify(GAtResult *result, gpointer user_data)
 		req.sms = sms;
 		req.store = store;
 		req.index = index;
+		req.expect_sr = expect_sr;
 
-		at_cmti_cpms_cb(TRUE, NULL, &req);
+		at_cmgr_cpms_cb(TRUE, NULL, &req);
 	} else {
 		char buf[128];
 		const char *incoming = storages[data->incoming];
@@ -466,26 +447,82 @@ static void at_cmti_notify(GAtResult *result, gpointer user_data)
 		req->sms = sms;
 		req->store = store;
 		req->index = index;
+		req->expect_sr = expect_sr;
 
-		sprintf(buf, "AT+CPMS=\"%s\",\"%s\",\"%s\"",
-			strstore, strstore, incoming);
+		snprintf(buf, sizeof(buf), "AT+CPMS=\"%s\",\"%s\",\"%s\"",
+				storages[store], storages[store], incoming);
 
-		g_at_chat_send(data->chat, buf, cpms_prefix, at_cmti_cpms_cb,
+		g_at_chat_send(data->chat, buf, cpms_prefix, at_cmgr_cpms_cb,
 				req, g_free);
 	}
+}
 
+static void at_cmti_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_sms *sms = user_data;
+	enum at_util_sms_store store;
+	int index;
+
+	if (at_util_parse_sms_index_delivery(result, "+CMTI:",
+						&store, &index) == FALSE)
+		goto error;
+
+	if (store != AT_UTIL_SMS_STORE_SM && store != AT_UTIL_SMS_STORE_ME)
+		goto error;
+
+	DBG("Got a CMTI indication at %s, index: %d", storages[store], index);
+	at_send_cmgr_cpms(sms, store, index, FALSE);
 	return;
 
-err:
+error:
 	ofono_error("Unable to parse CMTI notification");
 }
 
+static void at_cdsi_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_sms *sms = user_data;
+	enum at_util_sms_store store;
+	int index;
+
+	if (at_util_parse_sms_index_delivery(result, "+CDSI:",
+						&store, &index) == FALSE)
+		goto error;
+
+	/* Some modems actually store status reports in SM, and not SR */
+	if (store != AT_UTIL_SMS_STORE_SR && store != AT_UTIL_SMS_STORE_SM &&
+			store != AT_UTIL_SMS_STORE_ME)
+		goto error;
+
+	DBG("Got a CDSI indication at %s, index: %d", storages[store], index);
+	at_send_cmgr_cpms(sms, store, index, TRUE);
+	return;
+
+error:
+	ofono_error("Unable to parse CDSI notification");
+}
+
 static void at_cmgl_done(struct ofono_sms *sms)
 {
 	struct sms_data *data = ofono_sms_get_data(sms);
 
-	if (data->incoming == MT_STORE && data->store == ME_STORE)
-		at_cmgl_set_cpms(sms, SM_STORE);
+	if (data->incoming == AT_UTIL_SMS_STORE_MT &&
+			data->store == AT_UTIL_SMS_STORE_ME) {
+		at_cmgl_set_cpms(sms, AT_UTIL_SMS_STORE_SM);
+		return;
+	}
+
+	g_at_chat_register(data->chat, "+CMTI:", at_cmti_notify, FALSE,
+				sms, NULL);
+	g_at_chat_register(data->chat, "+CMT:", at_cmt_notify, TRUE,
+				sms, NULL);
+	g_at_chat_register(data->chat, "+CDS:", at_cds_notify, TRUE,
+				sms, NULL);
+	g_at_chat_register(data->chat, "+CDSI:", at_cdsi_notify, FALSE,
+				sms, NULL);
+
+	/* We treat CMGR just like a notification */
+	g_at_chat_register(data->chat, "+CMGR:", at_cmgr_notify, TRUE,
+				sms, NULL);
 }
 
 static void at_cmgl_notify(GAtResult *result, gpointer user_data)
@@ -501,8 +538,6 @@ static void at_cmgl_notify(GAtResult *result, gpointer user_data)
 	int status;
 	char buf[16];
 
-	dump_response("at_cmgl_notify", TRUE, result);
-
 	g_at_result_iter_init(&iter, result);
 
 	while (g_at_result_iter_next(&iter, "+CMGL:")) {
@@ -524,7 +559,7 @@ static void at_cmgl_notify(GAtResult *result, gpointer user_data)
 
 		hexpdu = g_at_result_pdu(result);
 
-		ofono_debug("Found an old SMS PDU: %s, with len: %d",
+		DBG("Found an old SMS PDU: %s, with len: %d",
 				hexpdu, tpdu_len);
 
 		if (strlen(hexpdu) > sizeof(pdu) * 2)
@@ -534,7 +569,7 @@ static void at_cmgl_notify(GAtResult *result, gpointer user_data)
 		ofono_sms_deliver_notify(sms, pdu, pdu_len, tpdu_len);
 
 		/* We don't buffer SMS on the SIM/ME, send along a CMGD */
-		sprintf(buf, "AT+CMGD=%d", index);
+		snprintf(buf, sizeof(buf), "AT+CMGD=%d", index);
 		g_at_chat_send(data->chat, buf, none_prefix,
 				at_cmgd_cb, NULL, NULL);
 	}
@@ -549,7 +584,7 @@ static void at_cmgl_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_sms *sms = user_data;
 
 	if (!ok)
-		ofono_debug("Initial listing SMS storage failed!");
+		DBG("Initial listing SMS storage failed!");
 
 	at_cmgl_done(sms);
 }
@@ -592,8 +627,8 @@ static void at_cmgl_set_cpms(struct ofono_sms *sms, int store)
 		req->sms = sms;
 		req->store = store;
 
-		sprintf(buf, "AT+CPMS=\"%s\",\"%s\",\"%s\"",
-			readwrite, readwrite, incoming);
+		snprintf(buf, sizeof(buf), "AT+CPMS=\"%s\",\"%s\",\"%s\"",
+				readwrite, readwrite, incoming);
 
 		g_at_chat_send(data->chat, buf, cpms_prefix, at_cmgl_cpms_cb,
 				req, g_free);
@@ -604,20 +639,9 @@ static void at_sms_initialized(struct ofono_sms *sms)
 {
 	struct sms_data *data = ofono_sms_get_data(sms);
 
-	g_at_chat_register(data->chat, "+CMTI:", at_cmti_notify, FALSE,
-				sms, NULL);
-	g_at_chat_register(data->chat, "+CMT:", at_cmt_notify, TRUE,
-				sms, NULL);
-	g_at_chat_register(data->chat, "+CDS:", at_cds_notify, TRUE,
-				sms, NULL);
-
-	/* We treat CMGR just like a notification */
-	g_at_chat_register(data->chat, "+CMGR:", at_cmgr_notify, TRUE,
-				sms, NULL);
-
 	/* Inspect and free the incoming SMS storage */
-	if (data->incoming == MT_STORE)
-		at_cmgl_set_cpms(sms, ME_STORE);
+	if (data->incoming == AT_UTIL_SMS_STORE_MT)
+		at_cmgl_set_cpms(sms, AT_UTIL_SMS_STORE_ME);
 	else
 		at_cmgl_set_cpms(sms, data->incoming);
 
@@ -681,7 +705,9 @@ static gboolean build_cnmi_string(char *buf, int *cnmi_opts,
 	const char *mode;
 	int len = sprintf(buf, "AT+CNMI=");
 
-	if (data->vendor == OFONO_VENDOR_QUALCOMM_MSM)
+	if (data->vendor == OFONO_VENDOR_QUALCOMM_MSM ||
+			data->vendor == OFONO_VENDOR_HUAWEI ||
+			data->vendor == OFONO_VENDOR_NOVATEL)
 		/* MSM devices advertise support for mode 2, but return an
 		 * error if we attempt to actually use it. */
 		mode = "1";
@@ -701,8 +727,22 @@ static gboolean build_cnmi_string(char *buf, int *cnmi_opts,
 	if (!append_cnmi_element(buf, &len, cnmi_opts[2], "20", FALSE))
 		return FALSE;
 
+	/*
+	 * Some manufacturers seem to have trouble with delivery via +CDS.
+	 * They report the status report properly, however refuse to +CNMA
+	 * ack it with error "CNMA not expected."  However, not acking it
+	 * sends the device into la-la land.
+	 */
+	if (data->vendor == OFONO_VENDOR_NOVATEL)
+		/* MSM devices advertise support for mode 2, but return an
+		 * error if we attempt to actually use it. */
+		mode = "2";
+	else
+		/* Sounds like 2 is the sanest mode */
+		mode = data->cnma_enabled ? "10" : "20";
+
 	/* Always deliver Status-Reports via +CDS or don't deliver at all */
-	if (!append_cnmi_element(buf, &len, cnmi_opts[3], "10", FALSE))
+	if (!append_cnmi_element(buf, &len, cnmi_opts[3], mode, FALSE))
 		return FALSE;
 
 	/* Don't care about buffering, 0 seems safer */
@@ -753,8 +793,6 @@ static void at_cnmi_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	gboolean supported = FALSE;
 	char buf[128];
 
-	dump_response("at_cnmi_query_cb", ok, result);
-
 	if (!ok)
 		goto out;
 
@@ -783,6 +821,12 @@ static void at_cnmi_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	if (build_cnmi_string(buf, cnmi_opts, data))
 		supported = TRUE;
 
+	/* support for ack pdu is not working */
+	if (data->vendor == OFONO_VENDOR_NOVATEL ||
+			data->vendor == OFONO_VENDOR_HUAWEI ||
+			data->vendor == OFONO_VENDOR_OPTION_HSO)
+		goto out;
+
 	if (data->cnma_enabled)
 		construct_ack_pdu(data);
 
@@ -807,8 +851,6 @@ static void at_cpms_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_sms *sms = user_data;
 	struct sms_data *data = ofono_sms_get_data(sms);
 
-	dump_response("at_cpms_set_cb", ok, result);
-
 	if (ok)
 		return at_query_cnmi(sms);
 
@@ -819,7 +861,7 @@ static void at_cpms_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		return at_sms_not_supported(sms);
 	}
 
-	g_timeout_add_seconds(1, set_cpms, sms);
+	data->timeout_source = g_timeout_add_seconds(1, set_cpms, sms);
 }
 
 static gboolean set_cpms(gpointer user_data)
@@ -830,10 +872,14 @@ static gboolean set_cpms(gpointer user_data)
 	const char *incoming = storages[data->incoming];
 	char buf[128];
 
-	sprintf(buf, "AT+CPMS=\"%s\",\"%s\",\"%s\"", store, store, incoming);
+	snprintf(buf, sizeof(buf), "AT+CPMS=\"%s\",\"%s\",\"%s\"",
+			store, store, incoming);
 
 	g_at_chat_send(data->chat, buf, cpms_prefix,
 			at_cpms_set_cb, sms, NULL);
+
+	data->timeout_source = 0;
+
 	return FALSE;
 }
 
@@ -842,8 +888,6 @@ static void at_cmgf_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_sms *sms = user_data;
 	struct sms_data *data = ofono_sms_get_data(sms);
 
-	dump_response("at_cmgf_set_cb", ok, result);
-
 	if (ok) {
 		data->retries = 0;
 		set_cpms(sms);
@@ -853,11 +897,11 @@ static void at_cmgf_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	data->retries += 1;
 
 	if (data->retries == MAX_CMGF_RETRIES) {
-		ofono_debug("Unable to enter PDU mode");
+		DBG("Unable to enter PDU mode");
 		return at_sms_not_supported(sms);
 	}
 
-	g_timeout_add_seconds(1, set_cmgf, sms);
+	data->timeout_source = g_timeout_add_seconds(1, set_cmgf, sms);
 }
 
 static gboolean set_cmgf(gpointer user_data)
@@ -867,6 +911,9 @@ static gboolean set_cmgf(gpointer user_data)
 
 	g_at_chat_send(data->chat, "AT+CMGF=0", cmgf_prefix,
 			at_cmgf_set_cb, sms, NULL);
+
+	data->timeout_source = 0;
+
 	return FALSE;
 }
 
@@ -877,8 +924,6 @@ static void at_cpms_query_cb(gboolean ok, GAtResult *result,
 	struct sms_data *data = ofono_sms_get_data(sms);
 	gboolean supported = FALSE;
 
-	dump_response("cpms_query_cb", ok, result);
-
 	if (ok) {
 		int mem = 0;
 		GAtResultIter iter;
@@ -918,12 +963,12 @@ static void at_cpms_query_cb(gboolean ok, GAtResult *result,
 
 		if (sm_supported[0] && sm_supported[1]) {
 			supported = TRUE;
-			data->store = SM_STORE;
+			data->store = AT_UTIL_SMS_STORE_SM;
 		}
 
 		if (me_supported[0] && me_supported[1]) {
 			supported = TRUE;
-			data->store = ME_STORE;
+			data->store = AT_UTIL_SMS_STORE_ME;
 		}
 
 		/* This seems to be a special case, where the modem will
@@ -931,13 +976,13 @@ static void at_cpms_query_cb(gboolean ok, GAtResult *result,
 		 * mem1
 		 */
 		if (mt_supported[2] && (sm_supported[0] || me_supported[0]))
-			data->incoming = MT_STORE;
+			data->incoming = AT_UTIL_SMS_STORE_MT;
 
 		if (sm_supported[2])
-			data->incoming = SM_STORE;
+			data->incoming = AT_UTIL_SMS_STORE_SM;
 
 		if (me_supported[2])
-			data->incoming = ME_STORE;
+			data->incoming = AT_UTIL_SMS_STORE_ME;
 	}
 out:
 	if (!supported)
@@ -953,8 +998,6 @@ static void at_cmgf_query_cb(gboolean ok, GAtResult *result,
 	struct sms_data *data = ofono_sms_get_data(sms);
 	gboolean supported = FALSE;
 
-	dump_response("cmgf_query_cb", ok, result);
-
 	if (ok) {
 		GAtResultIter iter;
 		int mode;
@@ -988,8 +1031,6 @@ static void at_csms_status_cb(gboolean ok, GAtResult *result,
 	struct sms_data *data = ofono_sms_get_data(sms);
 	gboolean supported = FALSE;
 
-	dump_response("csms_status_cb", ok, result);
-
 	if (ok) {
 		GAtResultIter iter;
 		int service;
@@ -1001,8 +1042,15 @@ static void at_csms_status_cb(gboolean ok, GAtResult *result,
 		if (!g_at_result_iter_next(&iter, "+CSMS:"))
 			goto out;
 
-		if (!g_at_result_iter_next_number(&iter, &service))
-			goto out;
+
+		if (data->vendor == OFONO_VENDOR_HUAWEI ||
+				data->vendor == OFONO_VENDOR_NOVATEL) {
+			g_at_result_iter_skip_next(&iter);
+			service = 0;
+		} else {
+			if (!g_at_result_iter_next_number(&iter, &service))
+				goto out;
+		}
 
 		if (!g_at_result_iter_next_number(&iter, &mt))
 			goto out;
@@ -1046,8 +1094,6 @@ static void at_csms_query_cb(gboolean ok, GAtResult *result,
 	int status_min, status_max;
 	char buf[128];
 
-	dump_response("csms_query_cb", ok, result);
-
 	if (!ok)
 		return at_sms_not_supported(sms);
 
@@ -1063,10 +1109,10 @@ static void at_csms_query_cb(gboolean ok, GAtResult *result,
 		if (status_min <= 1 && 1 <= status_max)
 			cnma_supported = TRUE;
 
-	ofono_debug("CSMS query parsed successfully");
+	DBG("CSMS query parsed successfully");
 
 out:
-	sprintf(buf, "AT+CSMS=%d", cnma_supported ? 1 : 0);
+	snprintf(buf, sizeof(buf), "AT+CSMS=%d", cnma_supported ? 1 : 0);
 	g_at_chat_send(data->chat, buf, csms_prefix,
 			at_csms_set_cb, sms, NULL);
 }
@@ -1096,13 +1142,16 @@ static void at_sms_remove(struct ofono_sms *sms)
 	if (data->cnma_ack_pdu)
 		g_free(data->cnma_ack_pdu);
 
+	if (data->timeout_source > 0)
+		g_source_remove(data->timeout_source);
+
 	g_free(data);
 }
 
 static struct ofono_sms_driver driver = {
-	.name = "atmodem",
-	.probe = at_sms_probe,
-	.remove = at_sms_remove,
+	.name		= "atmodem",
+	.probe		= at_sms_probe,
+	.remove		= at_sms_remove,
 	.sca_query	= at_csca_query,
 	.sca_set	= at_csca_set,
 	.submit		= at_cmgs,
